//
//  NTAppDelegate.m
//  NTIAPTool
//
//  Created by tzqiang on 02/09/2023.
//  Copyright (c) 2023 tzqiang. All rights reserved.
//

#import "NTAppDelegate.h"
#import "NTIAPTool.h"

@implementation NTAppDelegate

// Demo 代码使用参考，以下注释内容可根据项目组实际的业务来处理
- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions
{
    [[NTIAPTool shareInstance] nt_addIAPObserver];
    [NTIAPTool shareInstance].failureBlock = ^(NTIAPTransactionError errorCode, NSString * _Nonnull errorMessage) {
        if (errorCode == NTIAPTransactionErrorTransactionNotFinished || errorCode == NTIAPTransactionErrorRestoreTransactionFailed) {
//            AnalysisTool.event("订单交易失败", attributes: ["type" : message])
        } else {
//            AnalysisTool.event("直接购买失败", attributes: ["type" : message])
        }
//        HUDTool.showHUD(message)
    };
    [NTIAPTool shareInstance].verifyTransactionReceiptHandler = ^(NTIAPReceiptParams * _Nonnull params, SKPaymentTransaction * _Nonnull transaction) {
        if (params.receipt.length == 0) {
//            AnalysisTool.event("订单交易失败", attributes: ["type" : "receipt 票据为空"])
        }
        [self checkReceiptToServerWith:[self mapParams:params] isKeychain:NO handeler:^(BOOL result) {
            if (result) {
                [[NTIAPTool shareInstance] nt_finishTransaction:transaction];
            }
        }];
    };
    [NTIAPTool shareInstance].verifyKeychainReceiptHandler = ^(NTIAPReceiptParams * _Nonnull params) {
        if (params.receipt.length == 0) {
//            AnalysisTool.event("订单交易失败", attributes: ["type" : "receipt 票据为空"])
        }
        [self checkReceiptToServerWith:[self mapParams:params] isKeychain:YES handeler:^(BOOL result) {
            if (result) {
                [self deleteKeychain];
            }
        }];
    };
    
    // Override point for customization after application launch.
    return YES;
}

- (NSDictionary *)mapParams:(NTIAPReceiptParams *)params {
    NSMutableDictionary *paramString = [NSMutableDictionary dictionary];
    paramString[@"receipt"] = params.receipt;
    paramString[@"transactionId"] = params.transactionId;
    paramString[@"productId"] = params.productId;
    return paramString;
}

/// 恢复购买
- (void)restoreIAP {
//    AnalysisTool.event(@"恢复购买")
    [[NTIAPTool shareInstance] nt_restoreWithHUDHandler:^{
//        HUDTool.alwaysShow("正在为您恢复购买...")
    } completedTransactionsHUDHandler:^{
//        HUDTool.showHUD("已处理完所有可恢复交易")
    }];
}

/// 购买订阅
- (void)buyWithProductId:(NSString *)productId {
//    AnalysisTool.event(@"发起购买订阅")
//    if MobClick.isJailbroken() {
//        AnalysisTool.event("直接购买失败", attributes: ["type" : "越狱设备不支持购买"])
//        HUDTool.showHUD("越狱设备不支持购买")
//        return
//    }
    [[NTIAPTool shareInstance] nt_buyWithProductId:productId hudHandler:^{
//        HUDTool.alwaysShow("正在购买...")
    }];
}

/// 持久化的票据进行恢复校验
- (void)checkKeychainReceipt {
    if ([[NTIAPTool shareInstance] nt_hasKeychainReceipt]) {
        //    keychainRetryCount = 0
            [[NTIAPTool shareInstance] nt_verifyKeychainReceiptHUDHandler:^{
        //        HUDTool.alwaysShow("正在为您恢复处理异常订阅订单...")
            }];
    }
}

/// 删除持久化的交易数据
- (void)deleteKeychain {
    [[NTIAPTool shareInstance] nt_deleteKeychainReceipt];
}

/// 与服务器校验票据信息
- (void)checkReceiptToServerWith:(NSDictionary *)params
                      isKeychain:(BOOL)isKeychain
                        handeler:(void (^)(BOOL result))handeler {
//    [NetworkTool post:@"/applePay/receipt"
//               params:params
//            successed:^(id object) {
//        HUDTool.showHUDSuccess("订阅成功!")
//        AnalysisTool.event("订阅成功")
//        handeler(true);
//        // 网络错误重试
//        networrkRetryCount = 0;
//        // keychain 重试
//        keychainRetryCount = 0;
//    } failed:^(NSError * error) {
//        handeler(false);
//        [self retryReceiptToServerWith:params isKeychain:isKeychain handeler:^(BOOL result) {
//            handeler(result);
//        }];
//    }];
}

/// 弹窗提示用户重试
- (void)retryReceiptToServerWith:(NSDictionary *)params
                      isKeychain:(BOOL)isKeychain
                        handeler:(void (^)(BOOL result))handeler {
//    NSInteger count = networrkRetryCount;
//    if (isKeychain) {
//        count = keychainRetryCount;
//    }
//    // 多次尝试
//    if (count < 3) {
//        if (isKeychain) {
//            keychainRetryCount += 1;
//            AnalysisTool.event("内购 keychain 重试", attributes: ["count" : keychainRetryCount])
//        } else {
//            networrkRetryCount += 1;
//            AnalysisTool.event("内购网络重试", attributes: ["count" : networrkRetryCount])
//        }
//
//        UIAlertController *alertVC = [UIAlertController alertControllerWithTitle:@"错误提示！" message:@"购买订单验证异常，是否要重试？" preferredStyle:UIAlertControllerStyleAlert];
//        UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:@"取消" style:UIAlertActionStyleCancel handler:^(UIAlertAction * _Nonnull action) {
//            HUDTool.hideHUD()
//        }];
//        UIAlertAction *confirmAction = [UIAlertAction actionWithTitle:@"重试" style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
//            [self checkReceiptToServerWith:params isKeychain:isKeychain handeler:^(BOOL result) {
//                handeler(result);
//            }];
//        }];
//    } else {
//        // 尝试 n 次后，上传收集日志成功后，再删除 keychain 缓存的交易数据
//        [NetworkTool post:@"/applePay/report"
//                   params:params
//                successed:^(id object) {
//            handeler(false)
//            [self deleteKeychain];
//        } failed:^(NSError * error) {
//            handeler(false);
//        }];
//    }
}

/// 跳转 App Store 订阅管理
- (void)openSubscriptions {
    [[NTIAPTool shareInstance] nt_openSubscriptions];
}

- (void)applicationWillResignActive:(UIApplication *)application
{
    // Sent when the application is about to move from active to inactive state. This can occur for certain types of temporary interruptions (such as an incoming phone call or SMS message) or when the user quits the application and it begins the transition to the background state.
    // Use this method to pause ongoing tasks, disable timers, and throttle down OpenGL ES frame rates. Games should use this method to pause the game.
}

- (void)applicationDidEnterBackground:(UIApplication *)application
{
    // Use this method to release shared resources, save user data, invalidate timers, and store enough application state information to restore your application to its current state in case it is terminated later.
    // If your application supports background execution, this method is called instead of applicationWillTerminate: when the user quits.
}

- (void)applicationWillEnterForeground:(UIApplication *)application
{
    // Called as part of the transition from the background to the inactive state; here you can undo many of the changes made on entering the background.
}

- (void)applicationDidBecomeActive:(UIApplication *)application
{
    // Restart any tasks that were paused (or not yet started) while the application was inactive. If the application was previously in the background, optionally refresh the user interface.
}

- (void)applicationWillTerminate:(UIApplication *)application
{
    // Called when the application is about to terminate. Save data if appropriate. See also applicationDidEnterBackground:.
}

@end
